@@ -26,7 +26,7 @@ implemented as an easyblock
 """
 import copy
 import os
-from vsc import fancylogger
+from vsc.utils import fancylogger
 
 from easybuild.framework.easyblock import EasyBlock
 from easybuild.framework.easyconfig import CUSTOM
