@@ -1,4 +1,3 @@
-#
 # Copyright 2014-2014 Ghent University
 #
 # This file is part of EasyBuild,
@@ -30,6 +29,7 @@ and allow for reproducable builds
 @author: Stijn De Weirdt (Ghent University)
 """
 import time
+from easybuild.tools.filetools import det_size
 from easybuild.tools.ordereddict import OrderedDict
 from easybuild.tools.systemtools import get_system_info
 from easybuild.tools.version import EASYBLOCKS_VERSION, FRAMEWORK_VERSION
@@ -48,7 +48,7 @@ def get_build_stats(app, start_time, command_line):
         ('easybuild-easyblocks_version', str(EASYBLOCKS_VERSION)),
         ('timestamp', int(time_now)),
         ('build_time', build_time),
-        ('install_size', app.det_installsize()),
+        ('install_size', det_size(app.installdir)),
         ('command_line', command_line),
         ('modules_tool', app.modules_tool.buildstats()),
     ])
