@@ -53,11 +53,11 @@ _log = fancylogger.getLogger('config', fname=False)
 
 # class constant to prepare migration to generaloption as only way of configuration (maybe for v2.X)
 SUPPORT_OLDSTYLE = True
-
+DEFAULT_OLDSTYLE_CONFIG_FILE = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'easybuild_config.py')
 
 DEFAULT_LOGFILE_FORMAT = ("easybuild", "easybuild-%(name)s-%(version)s-%(date)s.%(time)s.log")
-
-
+DEFAULT_MNS = 'EasyBuildMNS'
+DEFAULT_MODULES_TOOL = 'EnvironmentModulesC'
 DEFAULT_PATH_SUBDIRS = {
     'buildpath': 'build',
     'installpath': '',
@@ -66,44 +66,79 @@ DEFAULT_PATH_SUBDIRS = {
     'subdir_modules': 'modules',
     'subdir_software': 'software',
 }
+DEFAULT_PREFIX = os.path.join(os.path.expanduser('~'), ".local", "easybuild")
+DEFAULT_REPOSITORY = 'FileRepository'
+DEFAULT_TMP_LOGDIR = tempfile.gettempdir()
 
+# utility function for obtaining default paths
+def mk_full_default_path(name, prefix=DEFAULT_PREFIX):
+    """Create full path, avoid '/' at the end."""
+    args = [prefix]
+    path = DEFAULT_PATH_SUBDIRS[name]
+    if path:
+        args.append(path)
+    return os.path.join(*args)
 
-DEFAULT_BUILD_OPTIONS = {
-    'aggregate_regtest': None,
-    'allow_modules_tool_mismatch': False,
-    'check_osdeps': True,
-    'filter_deps': None,
-    'cleanup_builddir': True,
-    'command_line': None,
-    'debug': False,
-    'dry_run': False,
-    'easyblock': None,
-    'experimental': False,
-    'force': False,
-    'github_user': None,
-    'group': None,
-    'hidden': False,
-    'ignore_dirs': None,
-    'modules_footer': None,
-    'only_blocks': None,
-    'optarch': None,
-    'recursive_mod_unload': False,
-    'regtest_output_dir': None,
-    'retain_all_deps': False,
-    'robot_path': None,
-    'sequential': False,
-    'set_gid_bit': False,
-    'silent': False,
-    'skip': None,
-    'skip_test_cases': False,
-    'sticky_bit': False,
-    'stop': None,
-    'suffix_modules_path': None,
-    'test_report_env_filter': None,
-    'umask': None,
-    'valid_module_classes': None,
-    'valid_stops': None,
-    'validate': True,
+# build options that have a perfectly matching command line option, listed by default value
+BUILD_OPTIONS_CMDLINE = {
+    None: [
+        'aggregate_regtest',
+        'dump_test_report',
+        'easyblock',
+        'filter_deps',
+        'from_pr',
+        'github_user',
+        'group',
+        'ignore_dirs',
+        'modules_footer',
+        'only_blocks',
+        'optarch',
+        'regtest_output_dir',
+        'skip',
+        'stop',
+        'suffix_modules_path',
+        'test_report_env_filter',
+        'testoutput',
+        'umask',
+    ],
+    False: [
+        'allow_modules_tool_mismatch',
+        'debug',
+        'experimental',
+        'force',
+        'hidden',
+        'robot',
+        'sequential',
+        'set_gid_bit',
+        'skip_test_cases',
+        'sticky_bit',
+        'upload_test_report',
+    ],
+    True: [
+        'cleanup_builddir',
+    ],
+}
+# build option that do not have a perfectly matching command line option
+BUILD_OPTIONS_OTHER = {
+    None: [
+        'build_specs',
+        'command_line',
+        'pr_path',
+        'robot_path',
+        'valid_module_classes',
+        'valid_stops',
+    ],
+    False: [
+        'dry_run',
+        'recursive_mod_unload',
+        'retain_all_deps',
+        'silent',
+        'try_to_generate',
+    ],
+    True: [
+        'check_osdeps',
+        'validate',
+    ],
 }
 
 
@@ -166,9 +201,7 @@ def map_to_newstyle(adict):
     res = {}
     for key, val in adict.items():
         if key in OLDSTYLE_NEWSTYLE_MAP:
-            newkey = OLDSTYLE_NEWSTYLE_MAP.get(key)
-            _log.deprecated("oldstyle key %s usage found, replacing with newkey %s" % (key, newkey), "2.0")
-            key = newkey
+            key = OLDSTYLE_NEWSTYLE_MAP[key]
         res[key] = val
     return res
 
@@ -220,7 +253,7 @@ class BuildOptions(FrozenDictKnownKeys):
     # singleton metaclass: only one instance is created
     __metaclass__ = Singleton
 
-    KNOWN_KEYS = DEFAULT_BUILD_OPTIONS.keys()
+    KNOWN_KEYS = [k for kss in [BUILD_OPTIONS_CMDLINE, BUILD_OPTIONS_OTHER] for ks in kss.values() for k in ks]
 
 
 def get_user_easybuild_dir():
@@ -229,12 +262,14 @@ def get_user_easybuild_dir():
     xdg_config_home = os.environ.get("XDG_CONFIG_HOME", os.path.join(os.path.expanduser('~'), ".config"))
     newpath = os.path.join(xdg_config_home, "easybuild")
 
-    if os.path.isdir(newpath):
-        return newpath
-    else:
+    # only issue deprecation warning/error if new path doesn't exist, but deprecated path does
+    if not os.path.isdir(newpath) and os.path.isdir(oldpath):
         _log.deprecated("The user easybuild dir has moved from %s to %s." % (oldpath, newpath), "2.0")
         return oldpath
 
+    # if neither exist, new path wins
+    return newpath
+
 
 def get_default_oldstyle_configfile():
     """Get the default location of the oldstyle config file to be set as default in the options"""
@@ -253,58 +288,15 @@ def get_default_oldstyle_configfile():
     else:
         # this should be easybuild.tools.config, the default config file is
         # part of framework in easybuild (ie in tool/..)
-        appPath = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
-        config_file = os.path.join(appPath, "easybuild_config.py")
-        _log.debug("Falling back to default config: %s" % config_file)
-
-    _log.deprecated("get_default_oldstyle_configfile oldstyle configfile %s used" % config_file, "2.0")
+        if os.path.exists(DEFAULT_OLDSTYLE_CONFIG_FILE):
+            config_file = DEFAULT_OLDSTYLE_CONFIG_FILE
+            _log.debug("Falling back to default config: %s" % config_file)
+        else:
+            config_file = None
 
     return config_file
 
 
-def get_default_oldstyle_configfile_defaults(prefix=None):
-    """
-    Return a dict with the defaults from the shipped legacy easybuild_config.py and/or environment variables
-        prefix: string, when provided, it used as prefix for the other defaults (where applicable)
-    """
-    if prefix is None:
-        prefix = os.path.join(os.path.expanduser('~'), ".local", "easybuild")
-
-    def mk_full_path(name):
-        """Create full path, avoid '/' at the end."""
-        args = [prefix]
-        path = DEFAULT_PATH_SUBDIRS[name]
-        if path:
-            args.append(path)
-        return os.path.join(*args)
-
-    # keys are the options dest
-    defaults = {
-        'config': get_default_oldstyle_configfile(),
-        'prefix': prefix,
-        'buildpath': mk_full_path('buildpath'),
-        'installpath': mk_full_path('installpath'),
-        'sourcepath': mk_full_path('sourcepath'),
-        'repository': 'FileRepository',
-        'repositorypath': {'FileRepository': [mk_full_path('repositorypath')]},
-        'logfile_format': DEFAULT_LOGFILE_FORMAT[:],  # make a copy
-        'tmp_logdir': tempfile.gettempdir(),
-        'moduleclasses': [x[0] for x in DEFAULT_MODULECLASSES],
-        'subdir_modules': DEFAULT_PATH_SUBDIRS['subdir_modules'],
-        'subdir_software': DEFAULT_PATH_SUBDIRS['subdir_software'],
-        'modules_tool': 'EnvironmentModulesC',
-        'module_naming_scheme': 'EasyBuildMNS',
-    }
-
-    # sanity check
-    if not defaults['repository'] in defaults['repositorypath']:
-        _log.error('Failed to get repository path default for default %s' % (defaults['repository']))
-
-    _log.deprecated("get_default_oldstyle_configfile_defaults", "2.0")
-
-    return defaults
-
-
 def get_default_configfiles():
     """Return a list of default configfiles for tools.options/generaloption"""
     return [os.path.join(get_user_easybuild_dir(), "config.cfg")]
@@ -321,10 +313,11 @@ def init(options, config_options_dict):
     Variables are read in this order of preference: generaloption > legacy environment > legacy config file
     """
     tmpdict = {}
-
     if SUPPORT_OLDSTYLE:
-
-        _log.deprecated('oldstyle init with modifications to support oldstyle options', '2.0')
+        if not os.path.samefile(options.config, DEFAULT_OLDSTYLE_CONFIG_FILE):
+            # only trip if an oldstyle config other than the default is used (via $EASYBUILDCONFIG or --config)
+            # we still need the oldstyle default config file to ensure legacy behavior, for now
+            _log.deprecated('use of oldstyle configuration file %s' % options.config, '2.0')
         tmpdict.update(oldstyle_init(options.config))
 
         # add the DEFAULT_MODULECLASSES as default (behavior is now that this extends the default list)
@@ -365,12 +358,46 @@ def init(options, config_options_dict):
     _log.debug("Config variables: %s" % variables)
 
 
-def init_build_options(build_options=None):
+def init_build_options(build_options=None, cmdline_options=None):
     """Initialize build options."""
-    # seed in defaults to make sure all build options are defined, and that build_option() doesn't fail on valid keys
-    bo = copy.deepcopy(DEFAULT_BUILD_OPTIONS)
+
+    active_build_options = {}
+
+    if cmdline_options is not None:
+        # building a dependency graph implies force, so that all dependencies are retained
+        # and also skips validation of easyconfigs (e.g. checking os dependencies)
+        retain_all_deps = False
+        if cmdline_options.dep_graph:
+            _log.info("Enabling force to generate dependency graph.")
+            cmdline_options.force = True
+            retain_all_deps = True
+
+        if cmdline_options.dep_graph or cmdline_options.dry_run or cmdline_options.dry_run_short:
+            _log.info("Ignoring OS dependencies for --dep-graph/--dry-run")
+            cmdline_options.ignore_osdeps = True
+
+        cmdline_build_option_names = [k for ks in BUILD_OPTIONS_CMDLINE.values() for k in ks]
+        active_build_options.update(dict([(key, getattr(cmdline_options, key)) for key in cmdline_build_option_names]))
+        # other options which can be derived but have no perfectly matching cmdline option
+        active_build_options.update({
+            'check_osdeps': not cmdline_options.ignore_osdeps,
+            'dry_run': cmdline_options.dry_run or cmdline_options.dry_run_short,
+            'recursive_mod_unload': cmdline_options.recursive_module_unload,
+            'retain_all_deps': retain_all_deps,
+            'validate': not cmdline_options.force,
+            'valid_module_classes': module_classes(),
+        })
+
     if build_options is not None:
-        bo.update(build_options)
+        active_build_options.update(build_options)
+
+    # seed in defaults to make sure all build options are defined, and that build_option() doesn't fail on valid keys
+    bo = {}
+    for build_options_by_default in [BUILD_OPTIONS_CMDLINE, BUILD_OPTIONS_OTHER]:
+        for default in build_options_by_default:
+            bo.update(dict([(opt, default) for opt in build_options_by_default[default]]))
+    bo.update(active_build_options)
+
     # BuildOptions is a singleton, so any future calls to BuildOptions will yield the same instance
     return BuildOptions(bo)
 
@@ -420,10 +447,9 @@ def install_path(typ=None):
         suffix = variables[key]
     else:
         # TODO remove default setting. it should have been set through options
-        _log.deprecated('%s not set in config, returning default' % key, "2.0")
-        defaults = get_default_oldstyle_configfile_defaults()
         try:
-            suffix = defaults[key]
+            suffix = DEFAULT_PATH_SUBDIRS[key]
+            _log.deprecated('%s not set in config, returning default: %s' % (key, suffix), "2.0")
         except:
             _log.error('install_path trying to get unknown suffix %s' % key)
 
@@ -467,10 +493,9 @@ def log_file_format(return_directory=False):
     if 'logfile_format' in variables:
         res = variables['logfile_format'][idx]
     else:
+        res = DEFAULT_LOGFILE_FORMAT[:][idx]  # purposely take a copy
         # TODO remove default setting. it should have been set through options
-        _log.deprecated('logfile_format not set in config, returning default', "2.0")
-        defaults = get_default_oldstyle_configfile_defaults()
-        res = defaults['logfile_format'][idx]
+        _log.deprecated('logfile_format not set in config, returning default: %s' % res, '2.0')
     return res
 
 
@@ -498,9 +523,8 @@ def get_build_log_path():
         return variables['tmp_logdir']
     else:
         # TODO remove default setting. it should have been set through options
-        _log.deprecated('tmp_logdir not set in config, returning default', "2.0")
-        defaults = get_default_oldstyle_configfile_defaults()
-        return defaults['tmp_logdir']
+        _log.deprecated('tmp_logdir not set in config, returning default: %s' % DEFAULT_TMP_LOGDIR, "2.0")
+        return DEFAULT_TMP_LOGDIR
 
 
 def get_log_filename(name, version, add_salt=False):
@@ -551,10 +575,10 @@ def module_classes():
     if 'moduleclasses' in variables:
         return variables['moduleclasses']
     else:
+        res = [x[0] for x in DEFAULT_MODULECLASSES]
         # TODO remove default setting. it should have been set through options
-        _log.deprecated('moduleclasses not set in config, returning default', "2.0")
-        defaults = get_default_oldstyle_configfile_defaults()
-        return defaults['moduleclasses']
+        _log.deprecated('moduleclasses not set in config, returning default: %s' % res, "2.0")
+        return res
 
 
 def read_environment(env_vars, strict=False):
@@ -569,7 +593,6 @@ def oldstyle_init(filename, **kwargs):
     Variables are read in this order of preference: CLI option > environment > config file
     """
     res = {}
-    _log.deprecated("oldstyle_init filename %s kwargs %s" % (filename, kwargs), "2.0")
 
     _log.debug('variables before oldstyle_init %s' % res)
     res.update(oldstyle_read_configuration(filename))  # config file
@@ -587,8 +610,6 @@ def oldstyle_read_configuration(filename):
     """
     Read variables from the config file
     """
-    _log.deprecated("oldstyle_read_configuration filename %s" % filename, "2.0")
-
     # import avail_repositories here to avoid cyclic dependencies
     # this block of code is going to be removed in EB v2.0
     from easybuild.tools.repository.repository import avail_repositories
@@ -606,8 +627,6 @@ def oldstyle_read_environment(env_vars=None, strict=False):
     Read variables from the environment
         - strict=True enforces that all possible environment variables are found
     """
-    _log.deprecated(('Adapt code to use read_environment from easybuild.tools.utilities '
-                     'and do not use oldstyle environment variables'), '2.0')
     if env_vars is None:
         env_vars = OLDSTYLE_ENVIRONMENT_VARIABLES
     result = {}
@@ -615,7 +634,7 @@ def oldstyle_read_environment(env_vars=None, strict=False):
         env_var = env_vars[key]
         if env_var in os.environ:
             result[key] = os.environ[env_var]
-            _log.deprecated("Found oldstyle environment variable %s for %s: %s" % (env_var, key, result[key]), "2.0")
+            _log.deprecated("Use of oldstyle environment variable %s for %s: %s" % (env_var, key, result[key]), '2.0')
         elif strict:
             _log.error("Can't determine value for %s. Environment variable %s is missing" % (key, env_var))
         else:
