@@ -31,4 +31,11 @@ from easybuild.framework.easyblock import EasyBlock
 
 class Toolchain(EasyBlock):
     """Dummy support for toolchains."""
-    pass
+    def configure_step(self):
+        pass
+    def build_step(self):
+        pass
+    def install_step(self):
+        pass
+    def sanity_check_step(self):
+        pass
